@@ -3,8 +3,10 @@
 #[macro_use]
 extern crate log;
 
+pub use lpc55::{uuid::Uuid, UuidSelectable};
+
 pub mod apps;
-pub use apps::App;
+pub use apps::{admin::App as Admin, Select};
 pub mod device;
 pub use device::{Device, Solo2};
 pub mod error;
@@ -12,6 +14,5 @@ pub use error::{Error, Result};
 pub mod firmware;
 pub use firmware::{Firmware, Version};
 pub mod pki;
-pub mod smartcard;
-pub use lpc55::{uuid::Uuid, UuidSelectable};
-pub use smartcard::Smartcard;
+pub mod transport;
+pub use transport::Transport;
